/*
 *   classifier.h
 *   $Id: classifier.h 171 2009-09-30 10:32:35Z ijuranek $
 */

#ifndef _ABON_CLASSIFIER_H_
#define _ABON_CLASSIFIER_H_

#include <iostream>

/// A single weak hypothesis.
/// 3x3 LRD feature, WaldBoost threshold and ptr to lookup table.
/// NOTE: 'alpha' should be indexed by values [-8..8], NOT [0..17]!!!
struct TStage
{
    // Feature and hypothesis parameters
    int x, y, w, h;
    char A, B, C;   ///< Rank indexes
    float theta_b;  ///< Wald_boost threshold
    float * alpha;  ///< Ptr to table with alphas

    // Additional parameters
    char szType;  // 2 * (h-1) + (w-1)
    char posType; // (4 * (y % 4) + (x % 4))
};


// Known types of Haar features
enum HaarType {
    H2V1=0, // Double horizontal
    H3V1,   // Triple horizontal
    H1V2,   // Double vertical
    H1V3,   // Triple vertical
    H2V2,   // Checker
    SRND,   // Surround
    numHaarType
};


struct THaarStage
{
    int x,y,w,h;
    HaarType tp;
    float min, max;
    int bins;
    float theta_b;
    float * alpha;

    // additional parameters (if any)
    unsigned offset; // offset from sample TLC
    unsigned yOffset; // offset between blocks in y direction
    unsigned sz; // w * h
};


// Types of stages that classifiers can hold
enum ClassifierType { UNKNOWN=0, LRD, LRP, LRP3, HAAR, LBP, numClassifierTypes };

/// Classifier structure.
/// Holds informations about a classifier - parameters,
/// weak hypotheses, alphas and convolutions.
struct TClassifier
{
    ClassifierType tp;            ///< Type of classifier
    unsigned stageCount;          ///< Total Number of stages
    unsigned evaluatedStageCount; ///< Max stages to evaluate
    unsigned alphaCount;          ///< Number of alphas per stage
    float threshold;              ///< Final classification threshold (typically 0.0f)
    unsigned width, height;       ///< Size of scanning window
    void * stage;                 ///< List of stages (vector of structures determined by the 'tp')
    float * alpha;                ///< List of alphas
};


/// Initialize classifier structure.
/// Links stages to alphas.
/// \param c Classifier to init
int initClassifier(TClassifier * c);

/// Load classifier from XML.
/// \param root Root element of the classifier
/// \returns Ptr to initialized classifier (no need to call initClassifier())
TClassifier * loadClassifierXML(const char * filename);

/// Disable early termination
/// Sets all thresholds to sufficiently low value
void disableClassifierWaldBoost(TClassifier * c);

void releaseClassifier(TClassifier ** c);

/// Export classifier to .h file
void exportClassifierHeader(TClassifier * c, std::ostream & str, const char * name);

void exportStaticClassifierCode(TClassifier * c, std::ostream & str, const char * name);

#ifdef __cplusplus
class ClassifierPtr
{
private :
    TClassifier *classifier;
public :
	ClassifierPtr(TClassifier *c) throw() : classifier(c) {}
	~ClassifierPtr() throw() { releaseClassifier(&classifier); }

    void set(TClassifier *c) throw() { classifier = c; }
};
#endif

#endif
